package com.antai.sport.http.server.server.api.business.club.service;

import com.antai.sport.http.server.common.exception.BusinessException;
import com.antai.sport.http.server.constants.ClubMemberType;
import com.antai.sport.http.server.constants.DeleteStatus;
import com.antai.sport.http.server.repository.club.entity.Club;
import com.antai.sport.http.server.repository.club.entity.ClubMember;
import com.antai.sport.http.server.repository.club.entity.ClubNotice;
import com.antai.sport.http.server.repository.club.mapper.ClubMapper;
import com.antai.sport.http.server.repository.club.mapper.ClubMemberMapper;
import com.antai.sport.http.server.repository.club.mapper.ClubNoticeMapper;
import com.antai.sport.http.server.server.api.business.club.converter.ClubConverter;
import com.antai.sport.http.server.server.api.business.club.dto.*;
import com.antai.sport.http.server.server.api.business.club.mapper.ClubBusinessMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.time.LocalDateTime;

@Service
public class ClubService {

    @Resource
    private ClubMapper clubMapper;
    @Resource
    private ClubMemberMapper clubMemberMapper;
    @Resource
    private ClubNoticeMapper clubNoticeMapper;

    @Resource
    private ClubBusinessMapper clubBusinessMapper;

    @Resource
    private ClubConverter clubConverter;

    /**
     * 创建俱乐部
     *
     * @param dto
     * @return
     */
    public Long create(CreateClubReqDTO dto) {
        if (clubBusinessMapper.checkChairman(dto.getCreateUser())) {
            throw new BusinessException("您已经是俱乐部会长，不能创建多个俱乐部");
        }
        checkClubNameRepeat(null, dto.getName());
        Club club = clubConverter.createDTOtoClub(dto, LocalDateTime.now());
        clubMapper.insert(club);
        return club.getId();
    }

    /**
     * 修改俱乐部信息
     *
     * @param dto
     */
    public void update(UpdateClubReqDTO dto) {
        checkClubNameRepeat(dto.getId(), dto.getName());
        Club club = clubConverter.updateDTOtoClub(dto, LocalDateTime.now());
        clubMapper.updateById(club);
    }

    /**
     * 校验俱乐部名称是否重复
     *
     * @param id
     * @param name
     */
    private void checkClubNameRepeat(Long id, String name) {
        if (clubBusinessMapper.checkClubNameRepeat(id, name)) {
            throw new BusinessException("俱乐部名称重复，请换一个名称试试");
        }
    }

    /**
     * 创建俱乐部通知
     *
     * @param dto
     * @return
     */
    public Long createClubNotice(CreateClubNoticeDTO dto) {
        this.checkClubChairman(dto.getClubId(), dto.getCreateUser());
        ClubNotice notice = clubConverter.createClubNoticeDTOToClubNotice(dto,LocalDateTime.now());
        clubNoticeMapper.insert(notice);
        return notice.getId();
    }

    /**
     * 修改俱乐部通知
     *
     * @param dto
     * @return
     */
    public void updateClubNotice(UpdateClubNoticeDTO dto) {
        this.checkClubChairman(dto.getClubId(), dto.getSportUserId());
        ClubNotice notice = clubConverter.updateClubNoticeDTOToClubNotice(dto,LocalDateTime.now());
        clubNoticeMapper.updateById(notice);
    }

    /**
     * 删除比赛通知
     * @param dto
     */
    public void deleteClubNotice(DeleteClubNoticeDTO dto){
        this.checkClubChairman(dto.getClubId(), dto.getSportUserId());
        ClubNotice notice = new ClubNotice();
        notice.setId(dto.getId());
        notice.setDeleted(1);
        clubNoticeMapper.updateById(notice);
    }


    /**
     * 校验通知操作用户是否为当前俱乐部会长
     *
     * @param clubId
     * @param sportUserId
     */
    private void checkClubChairman(Long clubId, Long sportUserId) {
        QueryWrapper<ClubMember> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("type", ClubMemberType.CHAIRMAN);
        queryWrapper.eq("deleted", DeleteStatus.UNDELETED);
        queryWrapper.eq("club_id", clubId);
        queryWrapper.eq("sport_user_id", sportUserId);
        ClubMember clubMember = clubMemberMapper.selectOne(queryWrapper);
        if (clubMember == null) {
            throw new BusinessException("您不是当前俱乐部会长，不能进行当前操作");
        }
    }
}
